﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Cvicenie05_pondelok
{
    public class Events
    {
        public enum OperationType
        {
            Add,
            Remove
        }

        public class CollectionModifiedEventArgs<T> : EventArgs
        {
            public OperationType Type { get; }

            public string ModifiedValue { get; }

            public CollectionModifiedEventArgs(T item, OperationType type)
            {
                Type = type;
                ModifiedValue = item.ToString();
            }
        }

        public class SimpleObservableCollection<T>
        {
            private readonly IList<T> _items = new List<T>();

            public int MyProperty { get; set; } = 10000;

            public void Add(T item)
            {
                _items.Add(item);
                OnCollectionModified(new CollectionModifiedEventArgs<T>(item, OperationType.Add));
            }

            public void Remove(T item)
            {
                if (_items.Contains(item))
                {
                    _items.Remove(item);
                    // ItemsModified?.Invoke(this, new CollectionModifiedEventArgs<T>(item, OperationType.Remove));
                    OnCollectionModified(e: new CollectionModifiedEventArgs<T>(item, OperationType.Remove));
                }
            }

            // Define at the top
            public event EventHandler<CollectionModifiedEventArgs<T>> ItemsModified;

            protected void OnCollectionModified(CollectionModifiedEventArgs<T> e)
                => ItemsModified?.Invoke(this, e);
        }

        public static void TestSolution()
        {
            var observableCollection = new SimpleObservableCollection<int>();

            observableCollection.ItemsModified += PrintModifications;
            observableCollection.ItemsModified += PrintModifications;
            observableCollection.ItemsModified += PrintModifications;

            observableCollection.Add(1);
            observableCollection.Add(2);
            observableCollection.Remove(2);
            observableCollection.Remove(2);

            observableCollection.ItemsModified -= PrintModifications;
            observableCollection.ItemsModified -= PrintModifications;
            observableCollection.ItemsModified -= PrintModifications;

            Console.ReadKey();
        }

        private static void PrintModifications(object sender, CollectionModifiedEventArgs<int> args)
        {
            Console.WriteLine(((SimpleObservableCollection<int>)sender).MyProperty);
            Console.WriteLine($"Operation: \"{args.Type}\" for element: {args.ModifiedValue}.");
        }
    }
}
